@@ -325,18 +325,16 @@ class MouseSelection(object):
         if event.button == 1:
             self.pressed = True
             x = event.x
-            start, end = self._selection.pixels()
+            pstart, pend = self._selection.pixels()
             # a double click resumes selection.
             if event.type == gtk.gdk._2BUTTON_PRESS:
                 self._selection.pin(x)
             # extend towards left
-            elif self._selection.selected() and near(start, x):
-                self._selection.pin(end)
-                self._selection.extend(x)
+            elif self._selection.selected() and near(pstart, x):
+                self._selection.move_start_to_pixel(x)
             # extend towards right
-            elif self._selection.selected() and near(end, x):
-                self._selection.pin(start)
-                self._selection.extend(x)
+            elif self._selection.selected() and near(pend, x):
+                self._selection.move_end_to_pixel(x)
             # start fresh selection
             else:
                 self._selection.pin(x)
