@@ -292,7 +292,7 @@ def get_libsndfile_version():
 MAJOR, MINOR, MICRO, PRERELEASE = get_libsndfile_version()
 if not(MAJOR == 1):
     raise Exception("audiolab expects major version %d of libsndfile" % 1)
-if not(MICRO == 17):
+if MICRO > 21:
     if PRERELEASE == 0: 
         prestr  = "No"
     else:
@@ -847,8 +847,8 @@ out of luck, because its implementation of POSIX open is broken)"""
         elif dtype == N.int16:
             res         = _SND.sf_readf_short(self.hdl, y, c_nframes)
         else:
-            RuntimeError("Sorry, only float, double, int and short read " + \
-                    "supported for now")
+            raise RuntimeError("Sorry, only float, double, int and short "
+                               "read supported for now")
 
         if not(res == nframes):
             msg = "Read %d frames, expected to read %d" % (res, nframes)
